'use strict';
var yeoman = require('yeoman-generator');
var chalk = require('chalk');
var yosay = require('yosay');
var assert = require('assert');
var _ = require('lodash');
var async = require('async');
var rd = require('rd');
var mkdirp = require('mkdirp');

_.extend(yeoman.Base.prototype, {
  tnpmInstall : function install(pkgs, options, cb) {
    return this.runInstall('tnpm', pkgs, options, cb);
  },
  installDependencies : function (options) {// 重载installDependencies
    var commands = [];
    var msg = {
      commands: [],
      template: _.template('\n\nI\'m all done. ' +
      '<%= skipInstall ? "Just run" : "Running" %> <%= commands %> ' +
      '<%= skipInstall ? "" : "for you " %>to install the required dependencies.' +
      '<% if (!skipInstall) { %> If this fails, try running the command yourself.<% } %>\n\n')
    };

    if (_.isFunction(options)) {
      options = {
        callback: options
      };
    }

    options = _.defaults(options || {}, {
      bower: true,
      npm: true,
      tnpm: true,
      skipMessage: false,
      callback: function () {}
    });

    if (options.tnpm) {
      msg.commands.push('tnpm install');
      commands.push(function (cb) {
        this.tnpmInstall(null, null, cb);
      }.bind(this));
    }

    if (options.npm) {
      msg.commands.push('npm install');
      commands.push(function (cb) {
        this.npmInstall(null, null, cb);
      }.bind(this));
    }

    if (options.bower) {
      msg.commands.push('bower install');
      commands.push(function (cb) {
        this.bowerInstall(null, null, cb);
      }.bind(this));
    }

    assert(msg.commands.length, 'installDependencies needs at least one of `npm` or `bower` to run.');

    if (!options.skipMessage) {
      var tplValues = _.extend({
        skipInstall: false
      }, this.options, {
        commands: chalk.yellow.bold(msg.commands.join(' && '))
      });
      this.log(msg.template(tplValues));
    }

    async.parallel(commands, options.callback);
  }
});

module.exports = yeoman.Base.extend({
  prompting: function () {
    // Have Yeoman greet the user.
    this.log(yosay(
      'Welcome to the prime ' + chalk.red('generator-xx') + ' generator!'
    ));

    var prompts = [{
      type: 'input',
      name: 'name',
      message: 'Your project name?',
      default: this.appname
    },{
      type: 'input',
      name: 'type',
      message: 'What`s your project?',
      default: 'default|mpa|react|vue'
    },{
      type: 'confirm',
      name: 'tnpm',
      message: 'If you want to use tnpm?'
    },{
      type: 'confirm',
      name: 'cdn',
      message: 'If you put static resource on cdn?'
    }];

    return this.prompt(prompts).then(function (props) {
      // To access props later use this.props.someAnswer;
      this.props = props;
    }.bind(this));
  },

  writing: function () {
    var files = ['boss.txt', 
                 'css/page.less', 'css/slices/',
                 'doc/',
                 'php/'
                ];
    var dests = JSON.parse(JSON.stringify(files));
    var that = this;

    switch(this.props.type){
      case 'mpa':
        files.push('js.mpa/');
        break;
      case 'react':
        files.push('js.react/');
        break;
      case 'vue':
        files.push('js.vue/');
        break;
      default:
        files.push('js/');
        break;
    }
    dests.push('js/');

    // package.json
    this.fs.copyTpl(
      this.templatePath('package.json'),
      this.destinationPath('package.json'), 
      {
        name : this.appname,
        tnpm : this.props.tnpm
      }
    );

    // gulpfile.json
    this.fs.copyTpl(
      this.templatePath('gulpfile.js'),
      this.destinationPath('gulpfile.js'), 
      {
        tnpm : this.props.tnpm,
        cdn : this.props.cdn
      }
    );

    // readme.md
    this.fs.copyTpl(
      this.templatePath('readme.md'),
      this.destinationPath('readme.md'), 
      {
        name : this.appname
      }
    );

    for(var i = 0; i < files.length; i++){
      // 深度遍历源目录，在目标位置创建目录结构
      rd.eachSync(this.templatePath(files[i]), function(f, s){
        if(!s.isFile()){
          f = f + '/';
          f = f.replace(that.templatePath(files[i]), that.destinationPath(dests[i]));// 替换为目标地址
          // console.log('src-->', that.templatePath(files[i]));
          // console.log('dest-->', that.destinationPath(dests[i]));
          // console.log('dir-->', f);
          // console.log('dest1-->', f.replace(that.templatePath(files[i]), that.destinationPath(dests[i])));
          // console.log('');
          mkdirp.sync(f);
        }
      });

      this.fs.copy(
        this.templatePath(files[i]),
        this.destinationPath(dests[i])
      );
    }

    // package.json
    // this.fs.copy(
    //   this.templatePath('package.js'),
    //   this.destinationPath('package.js')
    // );
    // // gulpfile.js
    // this.fs.copy(
    //   this.templatePath('gulpfile.js'),
    //   this.destinationPath('gulpfile.js')
    // );
  },

  install: function () {
    this.installDependencies({
      tnpm : this.props.tnpm,
      npm : !this.props.npm,
      bower : !this.props.npm
    });
  }
});
