var path = require('path');
var util = require('util');
var autoprefixer = require('autoprefixer-core');
var ExtractTextPlugin = require('extract-text-webpack-plugin');
var webpack = require('webpack');
var pkg = require('./package.json');

// 单js/css资源名
var jsBundle = path.join('js', util.format('%s.%s.js', pkg.name , pkg.version));
var cssBundle = path.join('css', util.format('%s.%s.css', pkg.name , pkg.version));

// 获取开发环境和生产环境
var DEBUG = (process.env.NODE_ENV === 'development');
var PROD = (process.env.NODE_ENV === 'production');

var cssLoader;
var htmlLoader = [
  'file-loader?name=[path][name].[ext]',
  'template-html-loader?' + [
    'raw=true',
    'engine=lodash',
    'version=' + pkg.version,
    'title=' + pkg.appName,
    'debug=' + DEBUG
  ].join('&')
].join('!');

var plugins = [
  new webpack.optimize.OccurenceOrderPlugin(),//给经常使用的模块分配最小长度的id
  new webpack.DefinePlugin({
    'process.env': {
      NODE_ENV: JSON.stringify('production')
    }
  })
];
if (DEBUG) {
  plugins.push(
    new webpack.HotModuleReplacementPlugin()//代码热替换
  );

  cssLoader = [
    'style-loader',// add css to the dom by injecting <style>
    'css-loader?sourceMap',// deal @import and url()
    'postcss-loader'// deal css 
  ].join('!');
} else {
  // 这个一定要加，否则——
  // Uncaught Error: locals[0] does not appear to be a `module` object with Hot Module replacement API enabled. 
  // You should disable react-transform-hmr in production by using `env` section in Babel configuration.
  plugins.push(
    new webpack.HotModuleReplacementPlugin()
  );

  plugins.push(
    new ExtractTextPlugin(cssBundle, {
      allChunks: true
    }),
    new webpack.optimize.UglifyJsPlugin(),// js混淆
    new webpack.optimize.DedupePlugin(),// 查找相等或近似的模块
    new webpack.NoErrorsPlugin()
  );

  cssLoader = ExtractTextPlugin.extract('style-loader', [
    'css-loader',
    'postcss-loader'
  ].join('!'));
}

var entry = {
  app: ['./js/router/index.js']
};
if (DEBUG) {
  entry.app.push(
    util.format(
      'webpack-dev-server/client?http://%s:%d',
      pkg.config.devHost,
      pkg.config.devPort
    )
  );
  entry.app.push('webpack/hot/dev-server');
}

module.exports = {
  // devtool: 'cheap-module-eval-source-map',
  cache: DEBUG,
  debug: DEBUG,
  target: 'web',
  entry: entry,
  output: {
    path: path.join(__dirname, 'dist'),
    filename: jsBundle,
    publicPath: '/',
    pathinfo: false
  },
  plugins: plugins,
  // plugins: [
  //   new webpack.optimize.OccurenceOrderPlugin(),
  //   new webpack.HotModuleReplacementPlugin()
  // ],
  module: {
    loaders: [
      {
        test: /\.js$/,
        loaders: [ 'babel' ],
        exclude: /node_modules/,
        include: __dirname
      },
      {
        test: /\.css?$/,
        loader: cssLoader,
        include: __dirname
      },
      {
        test: /\.s?html$/,
        loader: htmlLoader
      },
      {
        test: /\.jpe?g$|\.gif$|\.png$|\.ico|\.svg$|\.woff$|\.ttf$/,
        loader: "url-loader?limit=8192"
      }
    ]
  },
  postcss: [
    autoprefixer
  ],
  resolve: {
    extensions: ['', '.js', '.json', '.jsx']
  },
  devServer: {
    contentBase: path.resolve(pkg.config.buildDir),
    hot: true,
    noInfo: false,
    inline: true,
    stats: { colors: true }
  }
}


// When inside Redux repo, prefer src to compiled version.
// You can safely delete these lines in your project.
// var reduxSrc = path.join(__dirname, '..', '..', 'src')
// var reduxNodeModules = path.join(__dirname, '..', '..', 'node_modules')
// var fs = require('fs')
// if (fs.existsSync(reduxSrc) && fs.existsSync(reduxNodeModules)) {
//   // Resolve Redux to source
//   module.exports.resolve = { alias: { 'redux': reduxSrc } }
//   // Our root .babelrc needs this flag for CommonJS output
//   process.env.BABEL_ENV = 'commonjs'
//   // Compile Redux from source
//   module.exports.module.loaders.push({
//     test: /\.js$/,
//     loaders: [ 'babel' ],
//     include: reduxSrc
//   })
// }
